# -*- coding: utf-8 -*-
"""
Module de tests pour tp_boyer_moore.py
"""

import doctest
import time
import random
from tp_boyer_moore import *

tests = (("anticonstitutionnellement", "elle"),
             ("abcdabefabg", "ab"),
             ("abcdefg", "g"),
             ('deaaadcebbdbdcc', 'bdcc'),
             ("abcdefg", "cdf")
        )


def random_tests(len_strg, len_substrg, nb_tests, proba, len_alpha):
    """
    Construction aléatoire d'un jeu de tests pour la recherche textuelle
    
    Parameters
    ----------
    len_strg : int
        longueur des chaines strg à générer
    len_substrg : int
        longueur des chaines substrg à générer
    nb_tests : int
        nombre de jeux de tests à renvoyer
    proba : float
        probabilité d'insérer la sous-chaine à rechercher dans la chaine
        Il est possible que la sous-chaine soit crée dans la chaine par hasard
    len_alpha : int
        nombre de caractères dans l'alphabet

    Returns
    -------
    res : list
        liste de tuples (strg, substrg) servant à effectuer des tests

    """
    res = []
    for i in range(nb_tests):
        strg = ""
        for j in range(len_strg):
            strg += chr(random.randint(97, 97 + len_alpha))
        substrg = ""
        for j in range(len_substrg):
            substrg += chr(random.randint(97, 97 + len_alpha))
        if random.random() < proba:
            indice = random.randint(0, len_strg-len_substrg)
            strg = strg[:indice] + substrg + strg[indice + len(substrg):]
        res.append((strg, substrg))
    return res
    


def check_validity(solvers, tests):
    "Vérifie que tous les solveurs trouvent les mêmes résultats sur les tests proposés"
    res = []
    message = ""
    for t in tests:
        res.append(solvers[0](*t))
    for s in solvers[1:]:
        for i in range(len(tests)):
            if s(*tests[i]) != res[i]:
                message += "Test failed on {} with solvers {} and {}".format(tests[i], solvers[0].__name__, s.__name__) + "\n"
    if message == "":
        print("Validity tests successful" + "\n")
    else:
        print(message)
       


def run_performance(solvers, tests, nb_repeat):
    "Compare la vitesse des différents solveurs sur les tests proposés"
    for s in solvers:
        start_time = time.time()
        for t in tests:
            for n in range(nb_repeat):
                s(*t)
        end_time = time.time()
        print("{} ran the tests in {} seconds".format(s.__name__, end_time - start_time))



def test_solvers(solvers, test_parameters = None):
    "Lance les tests de validité et de performance sur plusieurs jeux de tests"
    check_validity(solvers, tests)
    if test_parameters == None:
        test_parameters = ((4000, 100, 50, 0.3, 80),
                           (4000, 15, 50, 0.3, 4),
                           (4000, 10, 50, 0.3, 30),
                           )
    for len_strg, len_substrg, nb_tests, proba, len_alpha in test_parameters:
        tests2 = random_tests(len_strg, len_substrg, nb_tests, proba, len_alpha)
        print("""\nstrg length : {}, substrg length : {}, number of tests : {}, probability of insering substrg in strg : {},
              number of symbols in the alphabet : {}\n""".format(len_strg, len_substrg, nb_tests, proba, len_alpha)) 
        check_validity(solvers, tests2)
        run_performance(solvers, tests2, 10)


# 2
def test_identique():
    tests = random_tests(20, 4, 10000, 0.9, 10)
    tests = [t + (random.randint(0,len(t[0])-len(t[1])), len(t[1])) for t in tests]
    solvers = [identique_0, identique]
    check_validity(solvers, tests)
    run_performance(solvers, tests, 20)

            
# 3
def test_recherche_naive():
    doctest.run_docstring_examples(recherche_naive, globals(), verbose=True)
   
# 4
def test_recherche_naive_2():
    test_solvers([recherche_naive, recherche_naive_2])

#5
def test_identique_1():
    tests = random_tests(20, 4, 10000, 0.9, 10)
    tests = [t + (random.randint(0,len(t[0])-len(t[1])), len(t[1])) for t in tests]
    solvers = [identique, identique_1]
    check_validity(solvers, tests)
    run_performance(solvers, tests, 20)  


def test_horspool_1():
    test_solvers([recherche_naive, horspool_1])
    

# 6
def test_identique_2():
    doctest.run_docstring_examples(identique_2, globals(), verbose=True)


def test_horspool_2():
    test_solvers([recherche_naive, horspool_1, horspool_2])
    

# 7
def test_premiere_table():
    doctest.run_docstring_examples(premiere_table, globals(), verbose=True)


# 8
def test_identique_3():
    doctest.run_docstring_examples(identique_3, globals(), verbose=True)


def test_horspool_3():
    test_solvers([recherche_naive, horspool_2, horspool_3])

# 9
def test_recherche_suffixe_0():
    doctest.run_docstring_examples(recherche_suffixe_0, globals(), verbose=True)


def test_recherche_suffixe_1():
    doctest.run_docstring_examples(recherche_suffixe_1, globals(), verbose=True)

def test_recherche_suffixe_2():
    doctest.run_docstring_examples(recherche_suffixe_2, globals(), verbose=True)


# 10

def test_seconde_table():
    doctest.run_docstring_examples(seconde_table, globals(), verbose=True)

def test_identique_4():
    doctest.run_docstring_examples(identique_4, globals(), verbose=True)


def test_boyer_moore():
    test_parameters = ((10000, 12, 50, 0.3, 26),
                       (200000, 100, 5, 0.3, 4), 
                       (10000, 50, 20, 0.3, 2),
                       )
    test_solvers([recherche_naive, horspool_3, boyer_moore], test_parameters)




